@@ -10,10 +10,10 @@ from funcy import lmap, map, lcat, join_with
 from django.utils.encoding import smart_str, force_text
 from django.core.exceptions import ImproperlyConfigured, EmptyResultSet
 from django.db import DEFAULT_DB_ALIAS
-from django.db.models import Model
+from django.db import models
 from django.db.models.manager import BaseManager
-from django.db.models.query import QuerySet
 from django.db.models.signals import pre_save, post_save, post_delete, m2m_changed
+from django.db.transaction import atomic
 
 # This thing reappeared in Django 3.0
 try:
@@ -59,7 +59,8 @@ def cache_thing(prefix, cache_key, data, cond_dnfs, timeout, dbs=(), precall_key
     )
 
 
-def cached_as(*samples, **kwargs):
+def cached_as(*samples, timeout=None, extra=None, lock=None, keep_fresh=False,
+                        key_func=func_cache_key):
     """
     Caches results of a function and invalidates them same way as given queryset(s).
     NOTE: Ignores queryset cached ops settings, always caches.
@@ -68,15 +69,8 @@ def cached_as(*samples, **kwargs):
     invalidated during the function call. This prevents prolonged caching of
     stale data.
     """
-    timeout = kwargs.pop('timeout', None)
-    extra = kwargs.pop('extra', None)
-    key_func = kwargs.pop('key_func', func_cache_key)
-    lock = kwargs.pop('lock', None)
-    keep_fresh = kwargs.pop('keep_fresh', False)
     if not samples:
         raise TypeError('Pass a queryset, a model or an object to cache like')
-    if kwargs:
-        raise TypeError('Unexpected keyword arguments %s' % ', '.join(kwargs))
 
     # If we unexpectedly get list instead of queryset return identity decorator.
     # Paginator could do this when page.object_list is empty.
@@ -84,9 +78,9 @@ def cached_as(*samples, **kwargs):
         return lambda func: func
 
     def _get_queryset(sample):
-        if isinstance(sample, Model):
+        if isinstance(sample, models.Model):
             queryset = sample.__class__.objects.filter(pk=sample.pk)
-        elif isinstance(sample, type) and issubclass(sample, Model):
+        elif isinstance(sample, type) and issubclass(sample, models.Model):
             queryset = sample.objects.all()
         else:
             queryset = sample
@@ -384,19 +378,22 @@ class QuerySetMixin(object):
         return objs
 
     def invalidated_update(self, **kwargs):
-        clone = self._clone().nocache()
+        clone = self._clone().nocache().select_related(None)
         clone._for_write = True  # affects routing
 
-        objects = list(clone)
-        rows = clone.update(**kwargs)
+        with atomic(using=clone.db):
+            objects = list(clone.select_for_update())
+            rows = clone.update(**kwargs)
+
+            # TODO: do not refetch objects but update with kwargs in simple cases?
+            # We use clone database to fetch new states, as this is the db they were written to.
+            # Using router with new_objects may fail, using self may return slave during lag.
+            pks = {obj.pk for obj in objects}
+            new_objects = self.model.objects.filter(pk__in=pks).using(clone.db)
 
-        # TODO: do not refetch objects but update with kwargs in simple cases?
-        # We use clone database to fetch new states, as this is the db they were written to.
-        # Using router with new_objects may fail, using self may return slave during lag.
-        pks = {obj.pk for obj in objects}
-        new_objects = self.model.objects.filter(pk__in=pks).using(clone.db)
         for obj in chain(objects, new_objects):
             invalidate_obj(obj, using=clone.db)
+
         return rows
 
 
@@ -449,6 +446,8 @@ class ManagerMixin(object):
             invalidate_obj(old, using=using)
         invalidate_obj(instance, using=using)
 
+        invalidate_o2o(sender, old, instance, using=using)
+
         # We run invalidations but skip caching if we are dirty
         if transaction_states[using].is_dirty():
             return
@@ -510,6 +509,19 @@ class ManagerMixin(object):
         return self.get_queryset().inplace().invalidated_update(**kwargs)
 
 
+def invalidate_o2o(sender, old, instance, using=DEFAULT_DB_ALIAS):
+    """Invoke invalidation for o2o reverse queries"""
+    o2o_fields = [f for f in sender._meta.fields if isinstance(f, models.OneToOneField)]
+    for f in o2o_fields:
+        old_value = getattr(old, f.attname, None)
+        value = getattr(instance, f.attname)
+        if old_value != value:
+            rmodel, rfield = f.related_model, f.remote_field.field_name
+            if old:
+                invalidate_dict(rmodel, {rfield: old_value}, using=using)
+            invalidate_dict(rmodel, {rfield: value}, using=using)
+
+
 def invalidate_m2m(sender=None, instance=None, model=None, action=None, pk_set=None, reverse=None,
                    using=DEFAULT_DB_ALIAS, **kwargs):
     """
@@ -549,7 +561,7 @@ def install_cacheops():
     Installs cacheops by numerous monkey patches
     """
     monkey_mix(BaseManager, ManagerMixin)
-    monkey_mix(QuerySet, QuerySetMixin)
+    monkey_mix(models.QuerySet, QuerySetMixin)
 
     # Use app registry to introspect used apps
     from django.apps import apps
